"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SuiPlatform = void 0;
const connect_sdk_1 = require("@wormhole-foundation/connect-sdk");
const client_1 = require("@mysten/sui.js/client");
const chain_1 = require("./chain");
const types_1 = require("./types");
const utils_1 = require("./utils");
/**
 * @category Sui
 */
class SuiPlatform extends connect_sdk_1.PlatformContext {
    static _platform = types_1._platform;
    constructor(network, _config) {
        super(network, _config ?? (0, connect_sdk_1.networkPlatformConfigs)(network, SuiPlatform._platform));
    }
    getRpc(chain) {
        if (chain in this.config)
            return new client_1.SuiClient({ url: this.config[chain].rpc });
        throw new Error("No configuration available for chain: " + chain);
    }
    getChain(chain) {
        if (chain in this.config)
            return new chain_1.SuiChain(chain, this);
        throw new Error("No configuration available for chain: " + chain);
    }
    static nativeTokenId(network, chain) {
        if (!SuiPlatform.isSupportedChain(chain))
            throw new Error(`invalid chain for ${types_1._platform}: ${chain}`);
        throw new Error("Not implemented");
    }
    static isNativeTokenId(network, chain, tokenId) {
        if (!SuiPlatform.isSupportedChain(chain))
            return false;
        if (tokenId.chain !== chain)
            return false;
        const native = this.nativeTokenId(network, chain);
        return native === tokenId;
    }
    static isSupportedChain(chain) {
        const platform = (0, connect_sdk_1.chainToPlatform)(chain);
        return platform === SuiPlatform._platform;
    }
    static async getDecimals(chain, rpc, token) {
        if ((0, connect_sdk_1.isNative)(token))
            return connect_sdk_1.decimals.nativeDecimals(SuiPlatform._platform);
        const tokenAddress = token.toString();
        try {
            const fields = await (0, utils_1.getObjectFields)(rpc, tokenAddress);
            if (fields && "decimals" in fields)
                return fields["decimals"];
        }
        catch { }
        const metadata = await rpc.getCoinMetadata({
            coinType: (0, types_1.getCoinTypeFromPackageId)(tokenAddress),
        });
        if (metadata === null)
            throw new Error(`Can't fetch decimals for token ${tokenAddress}`);
        return metadata.decimals;
    }
    static async getBalance(chain, rpc, walletAddr, token) {
        if (token === "native") {
            const { totalBalance } = await rpc.getBalance({
                owner: walletAddr,
            });
            return BigInt(totalBalance);
        }
        const { totalBalance } = await rpc.getBalance({
            owner: walletAddr,
            coinType: token.toString(),
        });
        return BigInt(totalBalance);
    }
    static async getBalances(chain, rpc, walletAddr, tokens) {
        throw new Error("Not implemented");
    }
    static async sendWait(chain, rpc, stxns) {
        const txhashes = [];
        for (const stxn of stxns) {
            console.log(stxn);
            const pendingTx = await rpc.executeTransactionBlock(stxn);
            await rpc.waitForTransactionBlock({ digest: pendingTx.digest });
            txhashes.push(pendingTx.digest);
        }
        return txhashes;
    }
    static async getLatestBlock(rpc) {
        return Number(await rpc.getLatestCheckpointSequenceNumber());
    }
    static async getLatestFinalizedBlock(rpc) {
        throw new Error("Not implemented");
    }
    static chainFromChainId(chainId) {
        const networkChainPair = connect_sdk_1.nativeChainIds.platformNativeChainIdToNetworkChain(SuiPlatform._platform, chainId);
        if (networkChainPair === undefined)
            throw new Error(`Unknown native chain id ${chainId}`);
        const [network, chain] = networkChainPair;
        return [network, chain];
    }
    static async chainFromRpc(rpc) {
        const result = await rpc.call("sui_getChainIdentifier", []);
        return this.chainFromChainId(result);
    }
}
exports.SuiPlatform = SuiPlatform;
//# sourceMappingURL=platform.js.map